<?php

/**
 * @file
 * Slack integration module rules functions.
 */

/**
 * Implements hook_rules_action_info().
 */
function amocrm_widget_slack_integration_rules_action_info() {
  $actions = array();
  $actions['amocrm_widget_slack_integration_send_message'] = array(
    'base' => 'amocrm_widget_slack_integration_rules_send_message_action',
    'label' => t('Slack send message'),
    'group' => t('Slack'),
    'parameter' => array(
      'webhook_url' => array(
        'type' => 'text',
        'label' => t('Webhook URL'),
        'description' => t("It will be using a webhook_url from slack module settings, if you don't enter it here."),
        'optional' => TRUE,
      ),
      'channel' => array(
        'type' => 'text',
        'label' => t('Channel'),
        'description' => t("It will be using a channel from slack module settings, if you don't enter channel here."),
        'optional' => TRUE,
      ),
      'username' => array(
        'type' => 'text',
        'label' => t('Username'),
        'description' => t("It will be using a username from slack module settings, if you don't enter username here."),
        'optional' => TRUE,
      ),
      'message' => array(
        'type' => 'text',
        'label' => t('Sending message'),
      ),
      'type_token' => array(
        'type' => 'text',
        'label' => t('Icon type'),
        'description' => t("Select needed token for define slackbot icon type, or select nothing to use default"),
        'optional' => TRUE,
      ),
      'emoji' => array(
        'type' => 'text',
        'label' => t('Emoji code'),
        'description' => t("It will be using a emoji code from slack module settings, if you don't enter username here."),
        'optional' => TRUE,
      ),
      'icon' => array(
        'type' => 'text',
        'label' => t('Enter icon url'),
        'description' => t("It will be using a icon url from slack module settings, if you don't enter username here."),
        'optional' => TRUE,
      ),
    ),
  );
  $actions['amocrm_widget_slack_integration_rules_create_channel_action'] = array(
    'base' => 'amocrm_widget_slack_integration_rules_create_channel_action',
    'label' => t('Create Slack channel'),
    'group' => t('Slack'),
    'parameter' => array(
      'name' => array(
        'type' => 'text',
        'label' => t('Channel name'),
        'description' => t('Name of the channel being created. Maximum length: 21 symbol'),
      ),
    ),
    'provides' => array(
      'amocrm_widget_slack_integration_created_channel' => array(
        'type' => 'amocrm_widget_slack_integration_created_channel',
        'label' => t('Channel created in slack'),
      ),
    ),
  );
  $actions['amocrm_widget_slack_integration_archive_channel'] = array(
    'base' => 'amocrm_widget_slack_integration_archive_channel',
    'label' => t('Archive Slack channel'),
    'group' => t('Slack'),
    'parameter' => array(
      'channel_id' => array(
        'type' => 'text',
        'label' => t('Channel id'),
        'description' => t('Channel id that you want to archive'),
      ),
    ),
  );
  return $actions;
}

/**
 * Rules action for sending a message to the Slack.
 */
function amocrm_widget_slack_integration_rules_send_message_action($webhook_url, $channel, $username, $message, $type_token = '', $emoji = '', $icon = '') {
  if (!$webhook_url) {
    $webhook_url = amocrm_widget_slack_integration_get_default_webhook_url();
  }
  if (!$channel) {
    $channel = amocrm_widget_slack_integration_get_default_channel();
  }
  if (!$username) {
    $username = amocrm_widget_slack_integration_get_default_username();
  }
  if (!$emoji) {
    $emoji = amocrm_widget_slack_integration_get_default_icon_emoji();
  }
  if (!$icon) {
    $icon = amocrm_widget_slack_integration_get_default_icon_icon();
  }
  $icon_options = array(
    'icon' => $icon,
    'emoji' => $emoji,
    'type' => $type_token,
  );
  amocrm_widget_slack_integration_send_message($webhook_url, $message, $channel, $username, $icon_options);
}

/**
 * Rules action for creating channel in Slack.
 */
function amocrm_widget_slack_integration_rules_create_channel_action($name) {
  $channel = amocrm_widget_slack_integration_create_channel($name);

  return $channel ? array('amocrm_widget_slack_integration_created_channel' => $channel) : array();
}
